@@ -74,7 +74,7 @@ def df2onehot(df, dtypes='pandas', y_min=None, perc_min_num=None, hot_only=True,
     # Determine Dtypes
     df, dtypes = set_dtypes(df, args['dtypes'], is_list=args['list_expand'], perc_min_num=args['perc_min_num'], verbose=args['verbose'])
     # If any column is a list, also expand the list!
-    df, dtypes = _expand_column_with_list(df, dtypes, args['verbose'])
+    df, dtypes = _deep_extract(df, dtypes, args['verbose'])
 
     # Make empty frames
     out_numeric = pd.DataFrame()
@@ -101,8 +101,8 @@ def df2onehot(df, dtypes='pandas', y_min=None, perc_min_num=None, hot_only=True,
             out_numeric[df.columns[i]] = out_numeric[df.columns[i]].astype('category')
             if verbose>=3: print('[df2onehot] >Working on %s%s[%.0f]' %(df.columns[i], makespaces, len(np.unique(integer_encoded)) ))
 
-            # Contains a single value
-            if len(np.unique(integer_encoded))<=1:
+            # Contains a single value or is bool
+            if (len(np.unique(integer_encoded))<=1) or (str(df.dtypes[i])=='bool'):
                 out_onehot[df.columns[i]] = integer_encoded.astype('Bool')
                 labx.append(df.columns[i])
             else:
@@ -147,67 +147,116 @@ def df2onehot(df, dtypes='pandas', y_min=None, perc_min_num=None, hot_only=True,
     return(out)
 
 
-# %%
-def _expand_column_with_list(df, dtypes, verbose=3):
-    # Check for any lists in dtypes
-    Icol = np.isin(dtypes,'list')
+# %% Convert str/float/int to type
+def _col2type(df, dtypes, idx):
+    gettype = dtypes[idx]
+    # Gather only the not NaN rows
+    Inan = df.iloc[:, idx].isna()
+    # Grap only unique elements from the combined set of lists
+    dfcol = df.iloc[~Inan.values, idx].copy()
+    # If any str, float or int elements is fount, convert to list
+    if gettype=='list':
+        typing = list(map(type, dfcol.values))
+        Iloc = np.logical_or(list(map(lambda x: isinstance(str(), x), typing)), np.logical_or(list(map(lambda x: isinstance(float(), x), typing)), list(map(lambda x: isinstance(int(), x), typing))))
+        if np.any(Iloc):
+            # Convert to string! This is required because the colnames also converts all to string.
+            dfcol.loc[Iloc] = list(map(lambda x: [str(x)], dfcol.loc[Iloc]))
+            df.iloc[dfcol.loc[Iloc].index,idx] = dfcol.loc[Iloc]
 
-    # If any
-    if np.any(Icol):
-        # Empty df
-        df2 = pd.DataFrame()
-        idxCol = np.where(Icol)[0]
-
-        # Expand columns with lists
-        for i in range(0,len(idxCol)):
-            if verbose>=3: print('[df2onehot] >Array-like column is detected: [%s]' %(df.columns[idxCol[i]]))
-            # Gather only the not NaN rows
-            Inan = df.iloc[:,idxCol[i]].isna()
-            # Grap only unique elements from the combined set of lists
-            dftmp = df.iloc[~Inan.values, idxCol[i]].copy()
-
-            # Make str, float and int elements of type list
-            typing = list(map(type, dftmp.values))
-            Iloc = np.logical_or(list(map(lambda x: isinstance(str(), x), typing)), np.logical_or(list(map(lambda x: isinstance(float(), x), typing)), list(map(lambda x: isinstance(int(), x), typing))))
-            if np.any(Iloc):
-                # Convert to string! This is required because the colnames also converts all to string.
-                dftmp.loc[Iloc] = list(map(lambda x: [str(x)], dftmp.loc[Iloc]))
-                df.iloc[dftmp.loc[Iloc].index,idxCol[i]] = dftmp.loc[Iloc]
-
-            # Get all unique elements
-            listvector = list(map(lambda x: list(x) , dftmp))
-            listvector = sum(listvector, [])
-            # All elements are converted to string!
-            colnames = np.unique(listvector)
-            # Lookup colname in the vector and make array
-            dftmp = df.iloc[:,idxCol[i]].apply(_findcol, cols=colnames)
-            arr = np.concatenate(dftmp).reshape((dftmp.shape[0], dftmp[0].shape[0]))
-            df1 = pd.DataFrame(index=np.arange(0,df.shape[0]), columns=colnames, data=arr, dtype='bool')
-            # Combine in one big matrix
-            if df2.empty:
-                df2 = df1.copy()
-                # df2 = pd.concat([df2.astype(bool), df1], axis=1)
-            else:
-                for colname in df1.columns:
-                    if np.any(np.isin(colname, df2.columns.values)):
-                        df2[colname] = np.logical_or(df2[colname], df1[colname])
-                    else:
-                        df2[colname] = df1[colname].copy()
-
-            # pd.merge(df_list_to_onehot, df1, on=df_list_to_onehot.columns.values).sum(axis=1)
-            # pd.merge(df_list_to_onehot, df1, how='inner')
-            # totlist = list(df_list_to_onehot.columns.values)+list(df1.columns.values)
-            # pd.concat([df_list_to_onehot, df1]).groupby(totlist).sum().reset_index()
-            # pd.merge(df_list_to_onehot, df1, on=totlist).sum(axis=1)
-            
-        # Drop columns that are expanded
-        df.drop(labels = df.columns[Icol].values, axis=1, inplace=True)
-        # Combine new one-hot-colums
-        df = pd.concat([df, df2], axis=1)
-
-    # Redo the typing
-    df, dtypes = set_dtypes(df, verbose=0)
+    # Get all unique elements
+    uifeat = _get_unique_elements(dfcol)
     # Return
+    return(df, uifeat)
+
+def _get_unique_elements(dfcol):
+    # Get all unique elements
+    listvector = list(map(lambda x: list(x) , dfcol))
+    listvector = sum(listvector, [])
+    # All elements are converted to string!
+    uifeat = np.unique(listvector)
+    return uifeat
+
+def _array2df(df, uifeat, idx):
+    # Lookup colname in the vector and make array
+    dfcol = df.iloc[:,idx].apply(_findcol, cols=uifeat)
+    arr = np.concatenate(dfcol).reshape((dfcol.shape[0], dfcol[0].shape[0]))
+    dfhot = pd.DataFrame(index=np.arange(0,df.shape[0]), columns=uifeat, data=arr, dtype='bool')
+    return dfhot
+
+def _concat(dftot, dfc):
+    # Combine in one big matrix
+    if dftot.empty:
+        dftot = dfc.copy()
+    else:
+        for colname in dfc.columns:
+            if np.any(np.isin(colname, dftot.columns.values)):
+                dftot[colname] = np.logical_or(dftot[colname], dfc[colname])
+            else:
+                dftot[colname] = dfc[colname].copy()
+    return dftot
+
+# %%
+def _deep_extract(df, dtypes, verbose=3):
+    # Check for any lists in dtypes
+    Ilist = np.isin(dtypes,'list')
+    Idict = np.isin(dtypes,'dict')
+    dftot1 = pd.DataFrame()
+    dftot2 = pd.DataFrame()
+    idxrem1 = []
+    idxrem2 = []
+
+    # Expand dict
+    if np.any(Idict):
+        idxCol = np.where(Idict)[0]
+        # Expand every columns that contains dict
+        for idx in idxCol:
+            if verbose>=3: print('[df2onehot] >%s column is detected: [%s]' %(dtypes[idx], df.columns[idx]))
+            try:
+                dfc = pd.DataFrame.from_records(df.iloc[:,idx])
+            except:
+                if verbose>=3: print('[df2onehot] >%s failed to process [%s].' %(dtypes[idx], df.columns[idx]))
+                dfc = df.iloc[:,idx].astype(str)
+
+            # Combine extracted columns into big dataframe
+            dftot1 = pd.concat([dftot1, dfc], axis=1)
+            # Add idx to remove
+            idxrem1.append(idx)
+
+    # Expand list
+    if np.any(Ilist):
+        idxCol = np.where(Ilist)[0]
+        # Expand every columns that contains either list
+        for idx in idxCol:
+            if verbose>=3: print('[df2onehot] >%s column is detected: [%s]' %(dtypes[idx], df.columns[idx]))
+            # Convert str/float/int to type
+            df, uifeat = _col2type(df, dtypes, idx)
+            # Convert column into onehot
+            dfc = _array2df(df, uifeat, idx)
+            # Combine hot-dataframes into one big dataframe
+            dftot2 = _concat(dftot2, dfc)
+            # Add idx to remove
+            idxrem2.append(idx)
+
+    # Drop columns that are expanded
+    idxrem = idxrem1+idxrem2
+    if len(idxrem)>0:
+        df.drop(labels = df.columns[idxrem].values, axis=1, inplace=True)
+        idxkeep = np.setdiff1d(np.arange(0, df.shape[1]), idxrem)
+        dtypes = np.array(dtypes)
+        dtypes = list(dtypes[idxkeep])
+        # Do the typing
+        dftot = pd.concat([dftot1, dftot2], axis=1)
+        dftot, dtypes1 = set_dtypes(dftot, verbose=3)
+        # Combine into dataframe
+        df = pd.concat([df, dftot], axis=1)
+        dtypes = dtypes + dtypes1
+        # Set dtypes for lists to bool
+        # dtypes = np.array(dtypes).astype('O')
+        # dtypes[idxrem2]='bool'
+        # dtypes = list(dtypes)
+
+    # Return
+    if verbose>=3: print('[df2onehot] >[%d] additional columns extracted by deep extract.' %(dftot1.shape[1]+dftot2.shape[1]))
     return(df, dtypes)
 
 
